/*******************************************************************************
*
*	TITLE:		clock2.c 
*
*	VERSION:	0.4 (Beta)                           
*
*	DATE:		18-Dec-2003
*
*	AUTHOR:		R. Kevin Watson
*				kevinw@jpl.nasa.gov
*
*	COMMENTS:	This demonstrates the use of a timer and associated interrupt.
*				In this example we'll setup a timer to generate an interrupt 
*				every millisecond (= 1000Hz rate). In response to this interrupt,
*				the microcontroller will execute a specific piece of code called
*				an interrupt handler (see user_routines_fast.c). This interrupt
*				handler will simply increment the global variables "msClock" and
*				"Clock". High-level user code can then access these variables to 
*				create sophisticated state machines and highly accurate sequencers.
*				We'll just keep the time.
*			
*
********************************************************************************
*
*	Change log:
*
*	DATE         REV  DESCRIPTION
*	-----------  ---  ----------------------------------------------------------
*	14-Dec-2003  0.1  RKW Original
*	16-Dec-2003  0.2  Accuracy bug: Changed the value of PR2 to 249 (was 250)
*	16-Dec-2003  0.3  Modified clock.c, changing the clock to 100ms (was 1s)
*	18-Dec-2003  0.4  Fixed 25 hour/day and 366 day/year bugs
*
*******************************************************************************/

#include "ifi_picdefs.h"
#include "printf_lib.h"
#include "clock2.h"

volatile unsigned long msClock = 0;	// 32-bit system clock running at 1KHz
									// msClock will roll-over every 49d 17h 2m 47s

volatile unsigned long Clock = 0;	// 32-bit system clock running at 10Hz
									// Clock will roll-over every 13.6 years

/*******************************************************************************
*
*	FUNCTION:		Initialize_Timer_2()
*
*	PURPOSE:		Initializes the timer 2 hardware.
*
*	CALLED FROM:	user_routines.c/User_Initialization()
*
*	PARAMETERS:		None
*
*	RETURNS:		Nothing
*
*	COMMENTS:		Place "#include "clock2.h" in the includes section
*					of user_routines.c then call Initialize_Timer_2() in
*					user_routines.c/User_Initialization().
*
*					Timer 2 documentation starts on page 141 of the data sheet.
*
*					Timer 2 will be setup with these parameters to generate
*					a 1000Hz interrupt rate:
*
*					1) The prescaler is set to divide the 10MHz system clock
*					by 4 (i.e., 1:4) so that TMR2 will be clocked at 2.5MHz.
*
*					2) The PR2 register is set to 249 so that the TMR2
*					register will roll-over (i.e., transition from 249 -> 0)
*					at a 10,000Hz rate (2.5MHz/250).
*
*					3) The postscaler is set to generate an interrupt every
*					tenth time the TMR2 register rolls-over (10,000Hz/10=1000Hz).
*
*					4) The timer 2 interrupt enable bit (TMR2IE) is set to
*					1 to enable the timer 2 interrupt.
*
*					5) Finally, timer 2 is allowed to run by setting the TMR2ON
*					bit to 1.
*
*******************************************************************************/
void Initialize_Timer_2(void)  
{
	TMR2 = 0;				// 8-bit timer 2 register (this is readable and writable)
							//
	PR2	= 249;				// timer 2 period register - timer 2 increments to this 
							// value then resets to zero on the next clock and starts
							// all over again
							//
	T2CONbits.T2OUTPS0 = 1;	// T2OUTPS3 T2OUTPS2 T2OUTPS1 T2OUTPS0
	T2CONbits.T2OUTPS1 = 0;	//    0        0        0        0		1:1 postscaler
	T2CONbits.T2OUTPS2 = 0;	//    0        0        0        1		1:2 postscaler
	T2CONbits.T2OUTPS3 = 1;	//    0        0        1        0		1:3 postscaler
							//    0        0        1        1		1:4 postscaler
							//    0        1        0        0		1:5 postscaler
							//    0        1        0        1		1:6 postscaler
							//    0        1        1        0		1:7 postscaler
							//    0        1        1        1		1:8 postscaler
							//    1        0        0        0		1:9 postscaler
							//    1        0        0        1		1:10 postscaler
							//    1        0        1        0		1:11 postscaler
							//    1        0        1        1		1:12 postscaler
							//    1        1        0        0		1:13 postscaler
							//    1        1        0        1		1:14 postscaler
							//    1        1        1        0		1:15 postscaler
							//    1        1        1        1		1:16 postscaler
							//
	T2CONbits.T2CKPS0 = 1;	// T2CKPS1  T2CKPS0
	T2CONbits.T2CKPS1 = 0;	//    0        0	1:1 prescaler (clock = 10MHz/each tick=100ns)
							//    0        1	1:4 prescaler (clock = 2.5MHz/each tick=400ns)
							//    1        x	1:16 prescaler (clock = 625KHz/each tick=1.6us) (T2CKPS0 doesn't matter)
							//
	PIE1bits.TMR2IE = 1;	// 0: disable timer 2 interrupt on PR2 match
							// 1: enable timer 2 interrupt on PR2 match
							//    if the prescaler is enabled (i.e., greater than 1:1), this
							//    match will occur n times (where n is the postscaler value)
							//    before an interrupt will be generated
							//
	IPR1bits.TMR2IP = 0;	// 0: timer 2 overflow interrupt is low priority (leave at 0 for IFI controllers)
							// 1: timer 2 overflow interrupt is high priority
							//
	T2CONbits.TMR2ON = 1;	// 0: timer 2 is disabled
							// 1: timer 2 is enabled (running)
}

/*******************************************************************************
*
*	FUNCTION:		Timer_2_Int_Handler()
*
*	PURPOSE:		If enabled, the timer 2 interrupt handler is called when the
*					TMR2 register matches the value stored in the PR2 register.
*
*	CALLED FROM:	user_routines_fast.c/InterruptHandlerLow()
*
*	PARAMETERS:		None
*
*	RETURNS:		Nothing
*
*	COMMENTS: 		The timer 2 module is documented in the PIC18F8520
*					data sheet starting on page 141.
*
*******************************************************************************/
void Timer_2_Int_Handler(void)
{
	// this function will be called when a timer 2 interrupt occurs

	static unsigned int divisor = 0;
 
	msClock++; // increment the millisecond clock

	// Do it this way so that we don't have to do anything as 
	// time consuming as division -- which is really, really
	// slow on an 8-bit microcontroller without hardware
	// support for division -- within an interrupt handler,
	// which must execute very fast.
 
	if (++divisor >= 100) // this can be adjusted up or down for better time accuracy
	{						
		divisor = 0;
		Clock++;	// increment the tenth of a second clock every 1/100th time through
	}
}

/*******************************************************************************
*
*	FUNCTION:		Display_Time()
*
*	PURPOSE:		Formats and sends the time to the console window
*					once a second.
*
*	CALLED FROM:	user_routines.c/Process_Data_From_Master_uP()
*
*	PARAMETERS:		None
*
*	RETURNS:		Nothing
*
*	COMMENTS:
*
*******************************************************************************/
void Display_Time(void)
{
	static unsigned long Old_Clock = 0;
	static unsigned char Tenths = 0;
	static unsigned char Seconds = 0;
	static unsigned char Minutes = 0;
	static unsigned char Hours = 0;
	static unsigned int Days = 0;

	if(Clock > Old_Clock) // only update the clock if it changes
	{
		Old_Clock = Clock;

		//
		// put code here that you want to run every tenth of a second
		//
		if(++Tenths >= 10)
		{
			Tenths = 0;
			Seconds++;
			//
			// put code here that you want to run every second
			//
		}
		if(Seconds >= 60)
		{
			Seconds = 0;
			Minutes++;
			//
			// put code here that you want to run every minute
			//
		}
		if (Minutes >= 60)
		{
			Minutes = 0;
			Hours++;
			//
			// put code here that you want to run every hour
			//
		}
		if (Hours >= 24)
		{
			Hours = 0;
			Days++;
			//
			// put code here that you want to run every day
			//
		}
		if (Days >= 365)
		{
			Days = 0;
			//
			// put code here that you want to run every year
			//
		}

		printf("Elapsed time = %03d:%02d:%02d:%02d.%d\n", (int)Days, (int)Hours, (int)Minutes, (int)Seconds, (int)Tenths);
	}
}
